# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz> 
"""
$Id: ForumPoll.py,v 1.14 2004/03/06 22:47:06 fafhrd91 Exp $
"""

import types, operator

from Products.Archetypes.public import *
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import View
from BTrees.Length import Length
from BTrees.IOBTree import IOBTree

from config import *
from ForumTopic import ForumTopic
from ForumPermissions import AddTopicReply, ModifyTopic

class ForumPoll(ForumTopic):
    """ forum poll topic based on MPoll product """
    meta_type = 'ForumPoll'
    archetype_name = 'Forum Poll'

    global_allow = 0
    filter_content_types = 1
    allowed_content_types = []
    content_icon = 'fcPoll_icon_noans.gif'

    schema = ForumTopic.schema + Schema((
        LinesField('answers', required=1, searchable=1,
                   write_permission=ModifyTopic,
                   widget=LinesWidget(description="A list of alternatives. One alternative per row.",
                                      label_msgid="label_answers",
                                      description_msgid="help_answers",
                                      i18n_domain="mpoll"),
                   ),
        ))
    
    actions = (
            {'id': 'view',
             'name': 'View',
             'action': 'string:$object_url/forum_topic',
             'permissions': (View,)
             },
            { 'id': 'edit',
              'name': 'Edit',
              'action': 'string:$object_url/forum_poll_edit',
              'permissions': (ModifyTopic,)
              },
            { 'id': 'reply',
              'name': 'Reply',
              'action': 'string:$object_url/forum_reply_form',
              'permissions': (AddTopicReply,)
              },
            { 'id': 'preview',
              'name': 'Preview',
              'action': 'string:$object_url/forum_message_preview',
              'permissions': (ModifyTopic,)
              },
            { 'id': 'delete',
              'name': 'Delete',
              'action': 'string:$object_url/forum_message_delete_form',
              'permissions': (ModifyTopic,)
              },         
            )


    security = ClassSecurityInfo()

    # poll
    topic_type = 3

    def __init__(self, oid, **kwargs):
        ForumTopic.__init__(self, oid, **kwargs)
        self._votes = IOBTree()

    security.declareProtected(View, 'CookedBody')
    def CookedBody(self):
        """ """
        msg = ForumTopic.CookedBody(self)
        return self.forum_poll_view(message=msg)

    security.declareProtected(View, 'isOpen')
    def isOpen(self):
        """ """
        wtool = getToolByName(self, 'portal_workflow')
        return wtool.getInfoFor(self,'review_state','') != 'locked'

    security.declareProtected(View, 'vote')
    def vote(self, answer):
        """Register a vote"""
        
        if not self.isOpen():
            return

        if not isinstance(answer, types.IntType):
            answer = int(answer)

        if not self._votes.has_key(answer):
            self._votes[answer] = Length()

        self._votes[answer].change(1)

    security.declareProtected(View, 'getVotes')
    def getVotes(self):
        """Result of this poll"""

        result = {'total': reduce(
            operator.add, [votes() for votes in self._votes.values()] + [0,0])}

        scores = []

        i = 1
        for answer in self.getAnswers():
            votes = 0
            percent = 0

            if self._votes.has_key(i):
                votes = self._votes[i]()
                percent = int(votes*100.0/result['total'])
            
            scores.append({'answer': answer,
                           'votes': votes,
                           'percent': percent})
            i = i+1
            
        result['scores'] = scores

        return result


registerType(ForumPoll)
