# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.kz> 
"""
$Id: UserPreferences.py,v 1.11 2004/03/24 07:13:03 fafhrd91 Exp $
"""

import random, traceback, time
from cPickle import loads, dumps

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base

from OFS.SimpleItem import SimpleItem
from Products.CMFCore.CMFCorePermissions import \
     View, ModifyPortalContent, AccessContentsInformation
from Products.CMFCore.utils import getToolByName, _checkPermission
from BTrees.OOBTree import OOBTree

from ForumCatalog import *
from ForumPermissions import *


class UserPreferences(SimpleItem):

    security = ClassSecurityInfo()

    security.declareProtected(View, 'get_user_role')
    def get_user_role(self, user, default=''):
        """ """
        user = getToolByName(self, 'portal_membership').getMemberById(user)
        try:
            if user:
                roles = user.getRoles()
                if ('Forum manager' in roles) or ('Manager' in roles):
                    return 'Manager'
                elif 'Forum reviewer' in roles:
                    return 'Reviewer'
                else:
                    return 'Member'
        except:
            pass

        return 'Anonymous'

    security.declareProtected(View, 'get_user_properties')
    def get_user_properties(self, user, REQUEST=None):
        """ """
        mtool = getToolByName(self, 'portal_membership')

        if REQUEST is not None:
            cache = REQUEST.get('forum_cache', {})
            if cache.has_key(user):
                return cache[user]

        data = {'fc_posts': 'n/a',
                'fc_emailtome': 0,
                'homepage': '',
                'fullname': str(user),
                'avatar': '',
                'anonymous': 1,
                }

        # maybe fix for LdapUserFolder
        if (str(user) == 'None') or not str(user):
            return data

        user = mtool.getMemberById(user)
        if user:
            data['anonymous'] = 0
            if user.getProperty('show_email', 0):
                email = user.getProperty('email', '')
                new_email = ''.join(['&#%s;'%ord(ch) for ch in email])
                data['email'] = new_email
                data['email_tag'] = '<a href="mailto:%s">%s</a>'%(new_email, new_email)

            if (user.getProperty('fc_emailtome', 0) and
                user.getProperty('email', '')):
                data['fc_emailtome'] = 1

            data['fc_posts'] = user.getProperty('fc_posts', 'n/a')
            data['homepage'] = user.getProperty('homepage', '')

            if user.getProperty('show_fullname', 0):
                data['fullname'] = user.getProperty('fullname','') or str(user)
                
            avatar = mtool.getPersonalPortrait(user.getId())
            if avatar.getId() != 'defaultUser.gif':
                data['avatar'] = avatar

            if REQUEST is not None:
                cache = REQUEST.get('forum_cache', {})
                cache[user] = data
                REQUEST.set('forum_cache', data)

        return data

    security.declareProtected(View, 'get_user_properties_full')
    def get_user_properties_full(self, user, REQUEST=None):
        """ """
        mtool = getToolByName(self, 'portal_membership')
        
        if REQUEST is not None:
            cache = REQUEST.get('forum_cache', {})
            if cache.has_key(user):
                return cache[user]

        data = {'fc_posts': 'n/a',
                'fc_emailtome': 0,
                'fc_forum_vm': 'flat',
                'fc_topic_vm': 'flat',
                'homepage': '',
                'fullname': str(user),
                'avatar': '',
                'anonymous': 1,
                }

        # maybe fix for LdapUserFolder
        if (str(user) == 'None') or not str(user):
            return data
        
        user = mtool.getMemberById(user)
        if user:
            data['anonymous'] = 0
            if user.getProperty('show_email', 0):
                email = user.getProperty('email', '')
                new_email = ''.join(['&#%s;'%ord(ch) for ch in email])
                data['email'] = new_email
                data['email_tag'] = '<a href="mailto:%s">%s</a>'%(new_email, new_email)
            if user.getProperty('show_im', 0):
                data['im_jabber'] = user.getProperty('im_jabber', '')
                data['im_yahoo'] = user.getProperty('im_yahoo', '')
                data['im_icq'] = user.getProperty('im_icq', '')
                data['im_msn'] = user.getProperty('im_msn', '')
                data['im_aim'] = user.getProperty('im_aim', '')
                data['im_ichat'] = user.getProperty('im_ichat', '')

            if (user.getProperty('fc_emailtome', 0) and
                user.getProperty('email', '')):
                data['fc_emailtome'] = 1

            data['fc_posts'] = user.getProperty('fc_posts', 'n/a')
            data['fc_forum_vm'] = user.getProperty('fc_forum_vm', 'flat')
            data['fc_topic_vm'] = user.getProperty('fc_topic_vm', 'flat')

            data['first_login_time'] = user.getProperty('first_login_time', '')
            data['last_login_time'] = user.getProperty('last_login_time', '')
            data['location'] = user.getProperty('location', '')
            data['occupation'] = user.getProperty('occupation', '')
            data['interests'] = user.getProperty('interests', '')
            data['homepage'] = user.getProperty('homepage', '')
            data['timezone'] = user.getProperty('timezone', '')

            if user.getProperty('show_fullname', 0):
                data['fullname'] = user.getProperty('fullname','') or str(user)

            avatar = mtool.getPersonalPortrait(user.getId())
            if avatar.getId() != 'defaultUser.gif':
                data['avatar'] = avatar

            if REQUEST is not None:
                cache = REQUEST.get('forum_cache', {})
                cache[user] = data
                REQUEST.set('forum_cache', data)

        return data

    security.declareProtected(View, 'get_read_status')
    def get_read_status(self, object, topic=0):
        """ """
        request = self.REQUEST

        if hasattr(request, '_read_status'):
            read_status = request._read_status
        else:
            mtool = getToolByName(self, 'portal_membership')
            if mtool.isAnonymousUser():
                return 1
            else:
                member = self.portal_membership.getAuthenticatedMember()
                try:
                    internal_data = loads(member.getProperty('fc_internal_data', ''))
                except:
                    internal_data = {}

                read_status = internal_data.get('read_status', {})
                read_status['last_login_time'] = member.getProperty('last_login_time')
                request._read_status = read_status

        data = read_status.setdefault('data', {})

        object_modified = object.modified()

        if read_status['last_login_time'] > object_modified:
            return 1

        if topic:
            topic_ob = object.getTopic()
            
            for ob in topic_ob.contentValues():
                if _checkPermission('View', ob):
                    status = self.get_read_status(ob)
                    if not status:
                        return 0


        if data.get(object.UID(), 0) > object_modified.timeTime():
            return 1

        return 0
    

    security.declareProtected(View, 'set_read_status')
    def set_read_status(self, object):
        """ """
        request = self.REQUEST

        mtool = getToolByName(self, 'portal_membership')
        if mtool.isAnonymousUser():
            return

        member = self.portal_membership.getAuthenticatedMember()
        try:
            last_login = member.getProperty('last_login_time')
            last_login_time = last_login.timeTime()
        except:
            return

        try:
            internal_data = loads(member.getProperty('fc_internal_data', ''))
        except:
            internal_data = {}

        read_status = internal_data.get('read_status', {})
        read_status['last_login_time'] = last_login
        data = read_status.get('data', {})

        object_uid = object.UID()
        object_modified = object.modified()
        object_modified_time = object_modified.timeTime()

        # check is it really need save access time
        if ((data.get(object_uid, 0) > object_modified_time) or
            (last_login > object_modified)):
            return
        
        data[object.UID()] = time.time()

        # clear data
        new_data = {}
        try:
            for uid, object_time in data.items():
                if object_time > last_login_time:
                    new_data[uid] = object_time
        except:
            pass

        # save data
        read_status['data'] = new_data
        internal_data['read_status'] = read_status
        member.setMemberProperties({'fc_internal_data': dumps(internal_data,1)})

        request._read_status = read_status


InitializeClass(UserPreferences)
