# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz>
"""
$Id: ui_get_forum_data.py,v 1.25 2004/03/26 10:34:40 fafhrd91 Exp $
"""

import traceback
from DateTime import DateTime
from Acquisition import aq_base, aq_inner, aq_parent
from Products.PageTemplates.GlobalTranslationService import getGlobalTranslationService
from Products.CMFCore.utils import getToolByName, _checkPermission
from Products.CMFPlone.PloneBatch import Batch
from Products.CMFCore.CMFCorePermissions import View, ReviewPortalContent
from Products.CMFBoard.ForumPermissions import *

def get_forum_menu(forum, portal, member):
    forumSchema = forum.Schema()
    mtool = portal.portal_membership
    request = forum.REQUEST
    session = request.get('SESSION', {})

    data = {}
    
    data['contact_admin'] = forumSchema.get('contact_admin').get(forum)
    data['forums_tree'] = get_forums_tree(forum)

    # new content creation links
    contents = {}
    if mtool.checkPermission(AddTopic, forum):
        contents['topic'] = 1

    if mtool.checkPermission(AddPoll, forum):
        contents['poll'] = 1

    if mtool.checkPermission(AddAnnouncement, forum):
        contents['announce'] = 1
    data['contents'] = contents

    # view mode
    view_mode_selectable = forumSchema.get('view_mode_selectable').get(forum)
    data['view_mode_selectable'] = view_mode_selectable
    if view_mode_selectable:
        name = '%s_viewmode'%forum.UID()
        view_mode = session.get(name, member.getProperty('fc_forum_vm', 'flat'))
        if view_mode not in ('flat','thread'):
            view_mode = 'flat'
        data['view_mode'] = view_mode
    else:
        view_mode = 'flat'
        data['view_mode'] = view_mode

    return data


def get_forum_data(forum, portal, member, basic=0):
    request = forum.REQUEST

    ctool = portal.portal_catalog
    mtool = portal.portal_membership
    trans = getGlobalTranslationService()

    timezone = member.getProperty('timezone','')
    forumTime = forum.forumTime
    forumSchema = forum.Schema()

    forum_page_size = forumSchema.get('forum_page_size').get(forum)
    topic_page_size = forumSchema.get('topic_page_size').get(forum)
    now = DateTime()
    isAnon = mtool.isAnonymousUser()
    session = request.get('SESSION', {})
    b_start = int(request.get('b_start', 0))

    f_catalog = forum.forum_catalog

    topics1 = f_catalog(review_state='admin',
                        sort_on='modified', sort_order='reverse')
    
    topics2 = f_catalog(review_state=['published', 'locked', 'disabled',
                                      'rejected','waitreview', 'announce'],
                        sticky_flag=1, sort_on='modified', sort_order='reverse')
    topics3 = f_catalog(review_state=['published', 'locked', 'disabled',
                                      'rejected','waitreview', 'announce'],
                        sticky_flag=0, sort_on='modified', sort_order='reverse',
                        sort_limit=forum_page_size+b_start)
    
    topics = topics1 + topics2 + topics3

    len(topics)

    batch = Batch(topics, forum_page_size, b_start, orphan=0)

    data = {}

    topics = list(batch)
    data['b_start'] = b_start
    data['batch'] = batch
    data['page_size'] = topic_page_size

    # cmfboard propsfo
    data['show_userinfo'] = forumSchema.get('show_userinfo').get(forum)
    data['show_wizards'] = forumSchema.get('show_wizards').get(forum)
    data['show_message_icons'] = forumSchema.get('show_message_icons').get(forum)
    data['allow_anon_names'] = forumSchema.get('allow_anon_names').get(forum)
    data['allow_signatures'] = forumSchema.get('allow_signatures').get(forum)

    # view mode
    view_mode_selectable = forumSchema.get('view_mode_selectable').get(forum)
    if view_mode_selectable:
        name = '%s_viewmode'%forum.UID()
        view_mode = session.get(name, member.getProperty('fc_forum_vm', 'flat'))
        if view_mode not in ('flat','thread'):
            view_mode = 'flat'
    else:
        view_mode = 'flat'

    # hits counter
    hits_counter = forumSchema.get('hits_counter').get(forum)
    data['hits_counter'] = hits_counter

    # parsing flag (emoticons)
    pf = forum.getField('parsing_flag').get(forum)
    if not pf:
        data['show_parsingflag'] = 1
    else:
        data['show_parsingflag'] = 0

    # read status
    read_status = forumSchema.get('read_status').get(forum)
    data['read_status'] = read_status

    try:
        session = request['SESSION']
    except:
        session = {}
        
    # topic status (new with replies, hot topic, etc)
    topic_hot = forumSchema.get('topic_hot').get(forum)
    topic_noreplies = forumSchema.get('topic_noreplies').get(forum)
    topic_nr_date = now - topic_noreplies

    # process topics
    topics_data = []
    data['topics_data'] = topics_data

    if basic:
        return data
    
    for brain in topics:
        rec = {}
        topic = brain.getObject()
        rec['topic'] = topic
        rec['brain'] = brain
        rec['created'] = forumTime(topic.created(), 1, timezone)

        url = topic.absolute_url()
        rec['url'] = url

        view_mode = topic.forum_get_view_mode(member)
        rec['view_mode'] = view_mode

        # actions
        actions = {}
        if _checkPermission(AddTopicReply, topic):
            actions['reply'] = '%s/forum_reply_form'%url
        if _checkPermission(DeleteTopic, topic):
            actions['delete'] = '%s/forum_message_delete_form'%url
        rec['actions'] = actions

        # find last reply
        topic_replies = ctool(portal_type='ForumMessage', path=brain.getPath(),
                              sort_on='created', sort_order='reverse',
                              sort_limit=1)
        if topic_replies:
            msg = topic_replies[0].getObject()
        else:
            msg = topic

        rec['msg'] = msg

        # basic attribues
        review_state = brain.review_state
        rec['review_state'] = review_state
        rec['sticky_flag'] = brain.sticky_flag

        rec['Title'] = topic.Title()
        rec['topicDesc'] = topic.Description()
        rec['Description'] = msg.Description()
        rec['topicIcon'] = topic.message_icon
        rec['modified'] = forumTime(brain.modified, 1, timezone)

        # message icon
        try:
            rec['msgIcon'] = getattr(forum.fcIcons, msg.message_icon)
        except:
            try:
                rec['msgIcon'] = getattr(forum.fcIcons, topic.message_icon)
            except:
                rec['msgIcon'] = '&nbsp;'

        # reply count
        replyCount = brain.replyCount
        rec['replyCount'] = replyCount

        # read status
        if read_status:
            rstatus = forum.get_read_status(topic, 1)
        else:
            rstatus = 0

        # hits counter
        if hits_counter:
            hits = topic.RawHits()
            rec['hits'] = hits
        else:
            hits = 0
            rec['hits'] = 0

        # topic status
        if review_state == 'announce':
            icon = 'announce'
            icon_title = 'Announcement'
        elif review_state == 'locked':
            if topic.portal_type == 'ForumPoll':
                icon = 'fcPoll_locked'
                icon_title = 'Locked poll'
            else:
                icon = 'fcTopic_locked'
                icon_title = 'Locked topic'
        else:
            s2 = 'Normal'
            s3 = ''
            if topic.portal_type == 'ForumPoll':
                icon = 'fcPoll_icon'
                s1 = 'poll'
            else:
                icon = 'fcTopic_icon'
                s1 = 'topic'

            if hits_counter:
                if not isAnon and read_status:
                    if hits >= topic_hot:
                        s2 = 'Hot'
                        if rstatus:
                            icon = '%s_hot_noans'%icon
                            s3 = 'without new replies'
                        else:
                            icon = '%s_hot'%icon
                            s3 = 'with new replies'
                    else:
                        if rstatus:
                            icon = '%s_noans'%icon
                            s3 = 'without new replies'
                else:
                    if hits >= topic_hot:
                        if brain.modified > topic_nr_date:
                            s2 = 'Hot'
                            if replyCount:
                                icon = '%s_hot'%icon
                                s3 = 'with new replies'
                            else:
                                icon = '%s_hot_noans'%icon
                                s3 = 'without new replies'
                        else:
                            icon = '%s_hot_noans'%icon
                            s2 = 'Hot'
                            s3 = 'without new replies'

                    else:
                        if (brain.modified < topic_nr_date) or not replyCount:
                            icon = '%s_noans'%icon
                            s3 = 'without new replies'
            icon_title = '%s %s %s'%(s2, s1, s3)
        try:
            icon_title = trans.translate('CMFBoard', icon_title, context=forum,
                                         default=icon_title)
            rec['icon'] = getattr(forum, '%s.gif'%icon).tag(alt=icon_title, title=icon_title)
        except:
            rec['icon'] = ''

        # pages links
        links = []
        if view_mode == 'flat':
            if replyCount >= topic_page_size:
                pages = int(round(float(replyCount)/topic_page_size + 0.5))
                
                if pages > 8:
                    for idx in (0,1,2):
                        links.append({'id':idx+1, 'url':'%s?%s'%(
                            url, 'b_start:int=%s'%(idx*topic_page_size))})
                        
                    result.append({'id':'...', 'url':''})
                    
                    for idx in range(pages)[-5:]:
                        links.append({'id':idx+1, 'url':'%s?%s'%(
                            url, 'b_start:int=%s'%(idx*topic_page_size))})
                else:
                    for idx in range(pages):
                        links.append({'id':idx+1, 'url':'%s?%s'%(
                            url, 'b_start:int=%s'%(idx*topic_page_size))})

        rec['pages_links'] = links

        # originator
        creator = brain.Creator
        rec['creator'] = creator
        if topic.isAnonymousCreator():
            rec['anonymous'] = 1
        else:
            rec['anonymous'] = 0
            rec['creator_info'] = forum.get_user_properties(creator)

        # link to last message
        msg_id = msg.getId()
        if view_mode in ('thread', 'full_thread'):
	    link = '%s#%s'%(msg.absolute_url(), msg_id)
        elif view_mode == 'nested':
            link = '%s#%s'%(url, msg_id)
        else:
            link = msg.absolute_url()
        rec['last_reply_link'] = link

        # last post user info
        if msg is topic:
            rec['last_reply_topic'] = 1

            rec['last_user'] = rec['creator']
            rec['last_user_isanon'] = rec['anonymous']
            if rec.has_key('creator_info'):
                rec['last_user_info'] = rec['creator_info']
        else:
            rec['last_reply_topic'] = 0
            rec['last_reply_title'] = msg.Title()
            rec['last_reply_created'] = forumTime(msg.created(), 1, timezone)
            
            creator = msg.Creator()
            rec['last_user'] = creator

            anonymous = msg.isAnonymousCreator()
            rec['last_user_isanon'] = anonymous
            if not anonymous:
                rec['last_user_info'] = forum.get_user_properties(creator)
        
        topics_data.append(rec)

        
    data.update(get_forum_menu(forum, portal, member))

    return data


def get_forums_tree(self):
    found = 0
    parent = self
    while not found:
        try:
            if parent.portal_type != 'ForumFolder':
                parent = aq_parent(aq_inner(parent))
            else:
                found = 1
        except:
            return []

    if parent.aq_parent.portal_type == 'ForumFolder':
        parent = aq_parent(aq_inner(parent))
   
    def add_rec(res, folder, mark1='', mark2='--'):
        if _checkPermission(View, folder):
            res.append({'title':folder.title_or_id(),
                        'disable':0, 'url':folder.absolute_url(),
                        'icon': folder.getIcon(),
                        'mark': mark1})

            for forum in folder.objectValues('ForumNB'):
                if _checkPermission(View, forum):
                    res.append(
                        {'title':forum.title_or_id(),
                         'disable':1, 'url': forum.absolute_url(),
                         'icon': forum.getIcon(),
                         'mark': mark2})

            for folder in folder.objectValues('ForumFolder'):
                add_rec(res, folder, mark1+'-', mark2+'--')

    res = []
    add_rec(res, parent)

    return res
