# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz>
"""
$Id: ui_get_message_data.py,v 1.19 2004/03/25 18:57:50 fafhrd91 Exp $
"""

from Acquisition import aq_base, aq_inner
from Products.PageTemplates.GlobalTranslationService \
     import getGlobalTranslationService
from Products.CMFCore.utils import getToolByName, _checkPermission
from Products.CMFPlone.PloneBatch import Batch
from Products.CMFCore.CMFCorePermissions import ReviewPortalContent
from Products.CMFBoard.ForumPermissions import *

from ui_get_topic_data import gen_thread
from ui_get_forum_data import get_forums_tree

def get_message_data(forum, message, portal, member, isActions=0, base=None):
    topic = message.getTopic()
    request = topic.REQUEST

    wtool = portal.portal_workflow
    mtool = portal.portal_membership
    trans = getGlobalTranslationService()

    timezone = member.getProperty('timezone','')
    forumTime = forum.forumTime
    forumSchema = forum.Schema()

    try:
        session = request['SESSION']
    except:
        session = {}

    data = {}

    topic_id = topic.getId()
    topic_uid = topic.UID()
    topic_url = topic.absolute_url()
    data['topic'] = topic
    data['topic_id'] = topic.getId()
    data['topic_url'] = topic_url
    data['page_size'] = forumSchema.get('topic_page_size').get(forum)

    isModerated = forum.moderated

    data['forums_tree'] = get_forums_tree(forum)

    replyable = _checkPermission(AddTopicReply, topic)
    data['replyable'] = replyable

    quickreply =  forum.quickReplyForm()
    data['quickreply'] =  quickreply

    # cmfboard propsfo
    data['show_userinfo'] = forumSchema.get('show_userinfo').get(forum)
    data['show_wizards'] = forumSchema.get('show_wizards').get(forum)
    data['show_message_icons'] = forumSchema.get('show_message_icons').get(forum)
    data['allow_anon_names'] = forumSchema.get('allow_anon_names').get(forum)
    data['allow_signatures'] = forumSchema.get('allow_signatures').get(forum)

    # view mode
    view_mode_selectable = forumSchema.get('view_mode_selectable').get(forum)
    data['view_mode_selectable'] = view_mode_selectable
    if view_mode_selectable:
        name = '%s_viewmode'%topic_uid
        view_mode = session.get(name, member.getProperty('fc_topic_vm', 'flat'))
        if view_mode not in ('flat','nested','thread','full_thread'):
            view_mode = 'flat'
        data['view_mode'] = view_mode
    else:
        view_mode = forum.getField('view_mode').get(forum)
        data['view_mode'] = view_mode

    # parsing flag (emoticons)
    pf = forum.getField('parsing_flag').get(forum)
    if not pf:
        data['show_parsingflag'] = 1
    else:
        data['show_parsingflag'] = 0

    # sort mode
    name = '%s_sortmode'%topic_uid
    sort_mode = session.get(name, member.getProperty('fc_topic_sm', 'default'))
    if sort_mode not in ('older_first','newer_first'):
        sort_mode = 'older_first'
    data['sort_mode'] = sort_mode

    # show userinfo
    show_userinfo = forumSchema.get('show_userinfo').get(forum)
    data['show_userinfo'] = show_userinfo

    # hits counter
    hits_counter = forumSchema.get('hits_counter').get(forum)
    data['hits_counter'] = hits_counter

    # basic attribues
    msg_id = message.getId()
    data['msg'] = message
    data['msg_id'] = msg_id
    data['msg_uid'] = message.UID()
    
    url = message.absolute_url()
    data['url'] = url
    
    title = message.Title()
    data['Title'] = title
    
    data['Body'] = message.CookedBody()
    data['created'] = forumTime(message.created(), 1, timezone)
    data['signature'] = message.getSignature()

    # icon
    icon = message.message_icon
    try:
        data['icon'] = getattr(forum.fcIcons, icon)
    except:
        data['icon'] = '&nbsp;&bull;'

    # review state
    review_state = wtool.getInfoFor(message, 'review_state', '')
    data['review_state'] = review_state

    # Creator
    creator = message.Creator()
    data['creator'] = creator

    isAnonymousCreator = message.isAnonymousCreator()
    data['anonymous'] = isAnonymousCreator

    if show_userinfo:
        creator_info = forum.get_user_properties(creator)

        # resize avatar
        avatar = creator_info.get('avatar', None)
        if avatar:
            creator_info['avatar_orig'] = avatar
            avatar_title = trans.translate(
                'CMFBoard', 'label_avatar', context=forum,
                default="Member's photo")
            creator_info['avatar'] = avatar.tag(
                width=75, height=100, alt=avatar_title)

        data['creator_info'] = creator_info
        data['creator_role'] = topic.get_user_role(creator,'Unavailable')

    # member menu
    if not isAnonymousCreator:
        info = {'id': 'mm_%s'%creator,
                'menus': [
            {'url': '%s/forum_user_profile?member_id=%s'%(topic_url, creator),
             'title': 'View public profile'},]
                }
        menus = info['menus']
        if ((getattr(forum, 'portal_messagetool', None) is not None) and
            mtool.getHomeFolder(creator)):
            menus.append({'url':'%s/forum_send_message_form?member_id=%s'%(url, creator),
                          'title':'Send a private message'},)
        if show_userinfo and creator_info['fc_emailtome']:
            menus.append({'url':'%s/forum_contact_member_form?member_id=%s'%(url, creator),
                          'title': 'Send email to member'})
        data['creator_menus'] = {creator: info}
    else:
        data['creator_menus'] = {}


    # actions
    if isActions:
        actions = {}

        if replyable:
            actions['reply'] = '%s/forum_reply_form'%url

        if (message.portal_type == 'ForumMessage'):
            if _checkPermission(ModifyReply, message):
                actions['edit'] = '%s/forum_message_edit'%url
            
            if _checkPermission(DeleteReply, message):
                actions['delete'] = '%s/forum_message_delete_form'%url

        else:
            if _checkPermission(ModifyTopic, message):
                actions['edit'] = '%s/forum_message_edit'%url

            if _checkPermission(DeleteTopic, message):
                actions['delete'] = '%s/forum_message_delete_form'%url

        if isModerated:
            transitions = wtool.getTransitionsFor(message)
            if transitions:
                actions['review_state'] = review_state
                actions['transitions'] = transitions

        data['actions'] = actions

    # quote link
    if replyable and quickreply and forumSchema.get('show_quote').get(forum):
        data['quote'] = 'quote(\'%s\',\'%s\',\'%s\');;return false;;'%(
            forum.forum_quote_format(message.EditableBody(),creator),
            message.UID(), (title[:3]=='Re:') and title or 'Re: '+title)
    else:
        data['quote'] = ''

    # image
    if hasattr(aq_base(message), 'image') and (message.image is not None):
        image = message.image
        size = image.getSize()
        if size:
            data['image'] = image
            data['image_url'] = '%s/image'%url
            data['image_size'] = '%0.1f Kb'%(float(size)/1024)

            if hasattr(message,'image_thumbnail'):
                data['thumbnail'] = message.image_thumbnail
                data['thumbnail_tag'] = message.image_thumbnail.tag(title='')

    # file
    if hasattr(aq_base(message), 'file') and (message.file is not None):
        file = message.file
        size = file.getSize()
        if size > 0:
            data['file'] = file
            data['file_url'] = '%s/file'%url
            data['file_size'] = '%0.1f Kb'%(float(size)/1024)

            info = {}
            try:
                info['filename'] = file.Filename()
            except:
                info['filename'] = ''

            try:
                icon = file.getMimetypeIcon()
                if icon:
                    info['icon'] = icon
                else:
                    info['icon'] = 'unknown.png'
            except:
                info['icon'] = 'unknown.png'

            info['icon'] = getattr(forum, info['icon'], '')
            data['file_info'] = info

    return data
