# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz>
"""
$Id: ui_get_topic_data.py,v 1.28 2004/03/25 18:57:50 fafhrd91 Exp $
"""

from __future__ import nested_scopes

from Acquisition import aq_base, aq_inner
from Products.PageTemplates.GlobalTranslationService \
     import getGlobalTranslationService
from Products.CMFCore.utils import getToolByName, _checkPermission
from Products.CMFPlone.PloneBatch import Batch
from Products.CMFCore.CMFCorePermissions import ReviewPortalContent
from Products.CMFBoard.ForumPermissions import *

from ui_get_forum_data import get_forums_tree

def get_topic_menu(forum, topic, message, portal, member):
    request = topic.REQUEST
    try:
        session = request['SESSION']
    except:
        session = {}

    forumSchema = forum.Schema()
    topic_uid = topic.UID()

    data = {}
    
    data['forums_tree'] = get_forums_tree(forum)

    # sort mode
    name = '%s_sortmode'%topic_uid
    sort_mode = session.get(name, member.getProperty('fc_topic_sm', 'default'))
    if sort_mode not in ('older_first','newer_first'):
        sort_mode = 'older_first'
    data['sort_mode'] = sort_mode

    # view mode
    view_mode_selectable = forumSchema.get('view_mode_selectable').get(forum)
    data['view_mode_selectable'] = view_mode_selectable
    if view_mode_selectable:
        name = '%s_viewmode'%topic_uid
        view_mode = session.get(name, member.getProperty('fc_topic_vm', 'flat'))
        if view_mode not in ('flat','nested','thread','full_thread'):
            view_mode = 'flat'
        data['view_mode'] = view_mode
    else:
        view_mode = forum.getField('view_mode').get(forum)
        data['view_mode'] = view_mode

    return data


def get_topic_data(forum, topic, message, portal, member):
    request = topic.REQUEST

    ctool = portal.portal_catalog
    mtool = portal.portal_membership
    wtool = portal.portal_workflow

    timezone = member.getProperty('timezone','')
    forumTime = forum.forumTime
    forumSchema = forum.Schema()
    
    data = {}
    topic_id = topic.getId()
    topic_uid = topic.UID()

    # load menu
    data.update(get_topic_menu(forum, topic, message, portal, member))

    # prev thread
    # need add read status
    topic_mod = topic.modified()
    fcatalog = forum.forum_catalog
    tp = fcatalog(modified={'query': (topic_mod,), 'range':'min'},
                  sort_on='modified', sort_limit=2)
    try:
        ob = tp[1].getObject()
        if ob.getId() == topic_id:
            data['prev_topic_url'] = None
        else:
            data['prev_topic_url'] = ob.absolute_url()
            data['prev_topic_title'] = ob.Title()
    except:
        pass
    
    # next thread
    tp = fcatalog(modified={'query': (topic_mod,), 'range':'max'},
                  sort_on='modified', sort_order='reverse', sort_limit=2)
    try:
        ob = tp[1].getObject()
        if ob.getId() == topic_id:
            data['next_topic_url'] = None
        else:
            data['next_topic_url'] = ob.absolute_url()
            data['next_topic_title'] = ob.Title()
    except:
        pass
    
    # New topic link
    if mtool.checkPermission(AddTopic, forum):
        data['new_topic_url'] = '%s/forum_add_topic_form'%forum.absolute_url()
    
    try:
        session = request['SESSION']
    except:
        session = {}

    # parsing flag (emoticons)
    pf = forum.getField('parsing_flag').get(forum)
    if not pf:
        data['show_parsingflag'] = 1
    else:
        data['show_parsingflag'] = 0

    sort_mode = data['sort_mode']
    view_mode = data['view_mode']

    data['topic'] = topic
    data['topic_id'] = topic.getId()
    data['topic_url'] = topic.absolute_url()

    isModerated = forum.moderated

    # replyable
    replyable = _checkPermission(AddTopicReply, topic)
    data['replyable'] = replyable

    # show userinfo
    show_userinfo = forumSchema.get('show_userinfo').get(forum)
    data['show_userinfo'] = show_userinfo

    # cmfboard properties
    data['show_wizards'] = forumSchema.get('show_wizards').get(forum)
    data['show_message_icons'] = forumSchema.get('show_message_icons').get(forum)
    data['allow_anon_names'] = forumSchema.get('allow_anon_names').get(forum)
    data['allow_signatures'] = forumSchema.get('allow_signatures').get(forum)

    # quickreply
    quickreply =  forum.quickReplyForm()
    data['quickreply'] = quickreply

    # hits counter
    hits_counter = forumSchema.get('hits_counter').get(forum)
    data['hits_counter'] = hits_counter

    if hits_counter:
        data['hits'] = topic.Hits()

    # creator_menus
    data['creator_menus'] = {}

    # local vars
    cr_roles = {}

    if view_mode == 'flat':
        data['replies_data'] = gen_flat(
            data, forum, topic, [], forumTime, timezone, cr_roles, replyable,
            isModerated, topic_id, quickreply,
            sort_mode, wtool, ctool, request, show_userinfo)
        
    elif view_mode ==  'full_thread':
        data['replies_data'] = gen_flat(
            data, forum, topic, [message], forumTime, timezone, cr_roles, replyable,
            isModerated, topic_id, quickreply,
            sort_mode, wtool, ctool, request, show_userinfo, 0)
        data['replies_data'][0]['tree'] = gen_thread(
            topic, topic, topic, forumTime, timezone)
        
    elif view_mode == 'thread':
        replies = [(m.created(), m) for m in topic.getBRefs('parent')]
        replies.sort()
        replies = [topic] + [m[1] for m in replies]
        
        replies_data = gen_flat(
            data, forum, topic, replies, forumTime, timezone, cr_roles, replyable,
            isModerated, topic_id, quickreply,
            sort_mode, wtool, ctool, request, show_userinfo, 0)
        replies_data[0]['tree'] = []

        for msg in replies_data[1:]:
            msg['tree'] = gen_thread(
                topic, msg['msg'], message, forumTime, timezone)
            
        data['replies_data'] = replies_data
            
    elif view_mode == 'nested':
        data['record'] = gen_nested(
            data, forum, topic, [message], forumTime, timezone, cr_roles,
            replyable, isModerated, topic_id, quickreply,
            sort_mode, wtool, ctool, request, show_userinfo)

    return data


def gen_flat(data, forum, topic, replies, forumTime, timezone, cr_roles,
             replyable, isModerated, topic_id, quickreply, sort_mode, 
             wtool, ctool, request, show_userinfo, isBrain=1):

    trans = getGlobalTranslationService()
    forumSchema = forum.Schema()

    # msgtool
    msgtool = getattr(forum, 'portal_messagetool', None)
    hasMsgtool = msgtool is not None
    mtool = getToolByName(forum, 'portal_membership')
    
    if not replies:
        topic_page_size = forumSchema.get('topic_page_size').get(forum)

        # replies
        replies = []
        if sort_mode == 'newer_first':
            replies = ctool(path='/'.join(topic.getPhysicalPath()),
                            sort_on='created', sort_order='reverse')
        else:
            replies = ctool(path='/'.join(topic.getPhysicalPath()), sort_on='created')

        b_start = int(request.get('b_start', 0))
        batch = Batch(replies, topic_page_size, b_start, orphan=0)

        replies = list(batch)
        data['b_start'] = b_start
        data['batch'] = batch
        data['page_size'] = topic_page_size

    # creator menus
    creator_menus = data['creator_menus']
    topic_url = data['topic_url']

    # process topics
    replies_data = []
    for brain in replies:
        rec = {}
        if isBrain:
            reply = brain.getObject()
        else:
            reply = brain

        msg_id = reply.getId()
        rec['msg'] = reply
        rec['msg_id'] = msg_id
        rec['msg_uid'] = reply.UID()
        rec['brain'] = brain

        url = reply.absolute_url()
        rec['url'] = url

        # basic attribues
        title = reply.Title()
        rec['Title'] = title
        rec['Body'] = reply.CookedBody()
        rec['signature'] = reply.getSignature()

        # icon
        icon = reply.message_icon
        try:
            rec['icon'] = getattr(forum.fcIcons, icon)
        except:
            rec['icon'] = '&nbsp;&bull;'

        # date
        if isBrain:
            rec['created'] = forumTime(brain.created, 1, timezone)
        else:
            rec['created'] = forumTime(brain.created(), 1, timezone)

        if isBrain:
            review_state = brain.review_state
        else:
            review_state = wtool.getInfoFor(reply, 'review_state', '')
            
        rec['review_state'] = review_state
            
        # Creator
        if isBrain:
            creator = brain.Creator
        else:
            creator = brain.Creator()

        rec['creator'] = creator
        
        anonymous = reply.isAnonymousCreator()
        rec['anonymous'] = anonymous

        if show_userinfo and not anonymous:
            creator_info = forum.get_user_properties(creator)

            avatar = creator_info.get('avatar', None)
            if avatar:
                creator_info['avatar_orig'] = avatar
                avatar_title = trans.translate(
                    'CMFBoard', 'label_avatar', context=forum,
                    default="Member's photo")
                creator_info['avatar'] = avatar.tag(
                    width=75, height=100, alt=avatar_title)
                
            rec['creator_info'] = creator_info

            if cr_roles.has_key(creator):
                rec['creator_role'] = cr_roles[creator]
            else:
                role = topic.get_user_role(creator,'Unavailable')
                rec['creator_role'] = role
                cr_roles[creator] = role

        # member menu
        if (show_userinfo and not creator_menus.has_key(creator) and not anonymous):
            info = {'id': 'mm_%s'%creator,
                    'menus': [
                {'url': '%s/forum_user_profile?member_id=%s'%(topic_url, creator),
                 'title': 'View public profile'},]
                }
            menus = info['menus']
            if hasMsgtool and mtool.getHomeFolder(creator):
                menus.append({'url':'%s/forum_send_message_form?member_id=%s'%(url, creator),
                              'title':'Send a private message'},)

            if creator_info['fc_emailtome']:
                menus.append({'url':'%s/forum_contact_member_form?member_id=%s'%(url, creator),
                              'title': 'Send email to member'})
            creator_menus[creator] = info

        # actions
        actions = {}

        if replyable:
            actions['reply'] = '%s/forum_reply_form'%url

        tp = reply.portal_type
        if (tp == 'ForumMessage'):
            if _checkPermission(ModifyReply, reply):
                actions['edit'] = '%s/forum_message_edit'%url

            if _checkPermission(DeleteReply, reply):
                actions['delete'] = '%s/forum_message_delete_form'%url

        else:
            if _checkPermission(ModifyTopic, reply):
                if tp == 'ForumPoll':
                    actions['edit'] = '%s/forum_poll_edit'%url
                elif tp == 'ForumContentTopic':
                    pass
                else:
                    actions['edit'] = '%s/forum_message_edit'%url

            if _checkPermission(DeleteTopic, reply):
                actions['delete'] = '%s/forum_message_delete_form'%url

        if isModerated:
            transitions = wtool.getTransitionsFor(reply)
            if transitions:
                actions['review_state'] = wtool.getInfoFor(reply,'review_state','')
                actions['transitions'] = transitions

        rec['actions'] = actions

        # quote link
        if replyable and quickreply and forumSchema.get('show_quote').get(forum):
            rec['quote'] = 'quote(\'%s\',\'%s\',\'%s\');;return false;;'%(
                forum.forum_quote_format(reply.EditableBody(),creator),
                reply.UID(),
                (title[:3]=='Re:') and title or 'Re: '+title)
        else:
            rec['quote'] = ''

        # image
        if hasattr(aq_base(reply), 'image') and (reply.image is not None):
            image = reply.image
            size = image.getSize()
            if size:
                rec['image'] = image
                rec['image_url'] = '%s/image'%url
                rec['image_size'] = '%0.1f Kb'%(float(size)/1024)

                if hasattr(reply,'image_thumbnail'):
                    rec['thumbnail'] = reply.image_thumbnail
                    rec['thumbnail_tag'] = reply.image_thumbnail.tag(title='')

        # file
        if hasattr(aq_base(reply), 'file') and (reply.file is not None):
            file = reply.file
            size = file.getSize()
            if size > 0:
                rec['file'] = file
                rec['file_url'] = '%s/file'%url
                rec['file_size'] = '%0.1f Kb'%(float(size)/1024)
            
                info = {}
                try:
                    info['filename'] = file.Filename()
                except:
                    info['filename'] = ''

                try:
                    icon = file.getMimetypeIcon()
                    if icon:
                        info['icon'] = icon
                    else:
                        info['icon'] = 'unknown.png'
                except:
                    info['icon'] = 'unknown.png'

                info['icon'] = getattr(forum, info['icon'], '')
                rec['file_info'] = info

        #dummy python:topic.set_read_status(msg);
        replies_data.append(rec)
    
    return replies_data


def gen_thread(topic, base, message, forumTime, timezone, remove_first=0):

    mesage_id = message.getId()

    def gen_level(msg, base_id):
        rec = {}

        icon = msg.message_icon
        try:
            rec['icon'] = getattr(topic.fcIcons, icon)
        except:
            rec['icon'] = '&nbsp;&bull;'

        msg_id = msg.getId()
        url = msg.absolute_url()

        rec['id'] = msg_id
        rec['url'] = url
        rec['read_status'] = topic.get_read_status(msg) and (msg_id != mesage_id)

        rec['title'] = msg.Title()
        rec['isHere'] = msg_id == mesage_id

        if hasattr(aq_base(msg), 'file') and (msg.file is not None):
            file = msg.file
            size = file.getSize()
            if size > 0:
                rec['file'] = file
                rec['file_url'] = '%s/file'%url
                rec['file_size'] = '%0.1f Kb'%(float(size)/1024)

                try:
                    icon = file.getMimetypeIcon()
                    if not icon:
                        icon = 'unknown.png'
                except:
                    icon = 'unknown.png'

                rec['file_icon'] = getattr(topic, icon, '')

        # image
        if hasattr(aq_base(msg), 'image') and (msg.image is not None):
            image = msg.image
            size = image.getSize()
            if size:
                rec['image'] = image
                rec['image_url'] = '%s/image'%url

        creator = msg.Creator()
        rec['creator'] = creator
        rec['date'] = forumTime(msg.created(),1,timezone)

        subtree = []
        for ch in msg.getChildren():
            subtree.append(gen_level(ch, base_id))

        rec['subtree'] = subtree
        return rec

    base_id = base.getId()

    data = gen_level(base, base.getId())

    if remove_first:
        return data['subtree']
    else:
        return [data]


def gen_nested(data, forum, topic, replies, forumTime, timezone, cr_roles, replyable,
               isModerated, topic_id, quickreply, sort_mode,
               wtool, ctool, request, show_userinfo):

    topic_url = data['topic_url']
    creator_menus = data['creator_menus']

    def gen_level(message, creator_menus, topic_url):
        data1 = {'topic_url': topic_url,
                 'creator_menus': creator_menus}

        d = gen_flat(data1, forum, topic, [message], forumTime, timezone,
                     cr_roles, replyable,
                     isModerated, topic_id, quickreply, sort_mode, wtool,
                     ctool, request, show_userinfo, 0)

        rec = d[0]
        
        subtree = []
        for msg in message.getChildren():
            subtree.append(gen_level(msg, creator_menus, topic_url))

        rec['subtree'] = subtree
        return rec


    return gen_level(topic, creator_menus, topic_url)
